eyg/ir/dag_json
Values
pub fn decode(
json: dynamic.Dynamic,
) -> Result(
#(tree.Expression(Nil), Nil),
List(decode.DecodeError),
)
Deprecated: This function is tied to Nil metadata, use the `decoder` directly instead
pub fn decoder(
meta: meta,
) -> decode.Decoder(#(tree.Expression(meta), meta))
create a decoder that will parse JSON source to an annotated tree with the given metadata.
pub fn from_block(
data: BitArray,
) -> Result(#(tree.Expression(Nil), Nil), json.DecodeError)
Deprecated: This function is tied to Nil metadata, use the `decoder` directly instead
pub fn to_block(data: #(tree.Expression(meta), meta)) -> BitArray
pub fn to_data_model(
tree: #(tree.Expression(meta), meta),
) -> json.Json
pub fn to_string(data: #(tree.Expression(meta), meta)) -> String
pub const vacant_cid: v1.Cid